<?php
namespace PME\Controllers\Cardapio;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager\Doctrine\NameProvider;

class ComensalEstimadoCardapio extends \PME\Services\TeknisaCrud {
    protected $crudService;
	protected $parameterBag;
	protected $dataSourceName;
	protected $connection;
	protected $newCode;
	protected $environment;
	protected $entityManager;
	protected $elaboracaoCardapio;
	protected $utilitiesCard;

    public function __construct($crudService, $parameterBag, $connection, $newCode, $environment, $entityManager, $elaboracaoCardapio, $utilitiesCard) {
        parent::__construct($crudService, $parameterBag);
        $this->connection = $connection;
        $this->newCode = $newCode;
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->elaboracaoCardapio = $elaboracaoCardapio;
        $this->utilitiesCard = $utilitiesCard;
	}
	
	public function update(DTO\Request\Row $request, DTO\Response $response) {
	    $row = $request->getParameters();
	    try {
	        $this->connection->beginTransaction();
	        foreach ($row['editedRows'] as $editedRow) {
	        	$this->updateComensal($editedRow, $editedRow['CDFILIAL']);	
	        	$this->updateComenCentral($editedRow);
	        }
	        $this->connection->commit();
	        
	        $response->addDataSet(new DataSet("update", array(true)));
	    } catch (\Exception $e) {
	    	$this->connection->rollback();
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	private function updateComensal($row, $cdfilial) {
	    $update = "UPDATE CARDIAPME SET QTFUNESTCPME = :QTFUNESTCPME, QTDOAESTCPME = :QTDOAESTCPME, QTCLIESTCPME = :QTCLIESTCPME, QTCOMESTCPME = :QTCOMESTCPME WHERE NRCARDPME = :P_NRCARDPME AND CDFILCENTPME = :P_CDFILCENTPME AND CDSERVFILI = :P_CDSERVFILI AND CDFILIAL = NVL(:P_CDFILIAL, CDFILCENTPME) AND DTCARDIAPME = :P_DTCARDIAPME";
	    $params = array(
            "QTFUNESTCPME"   => $row['QTFUNESTCPME'],
            "QTDOAESTCPME"   => $row['QTDOAESTCPME'],
            "QTCLIESTCPME"   => $row['QTCLIESTCPME'],
            "QTCOMESTCPME"   => $row['QTCOMESTCPME'],
            "P_NRCARDPME"    => $row['NRCARDPME'],
            "P_CDFILCENTPME" => $row['CDFILCENTPME'],
            "P_CDSERVFILI"   => $row['CDSERVFILI'],
            "P_CDFILIAL"     => $cdfilial,
            "P_DTCARDIAPME"  => $row['DTCARDIAPME']
        );
        $stmt = $this->connection->prepare($update);
        $stmt->execute($params);
	}
	
	public function updateComenCentral($row) {
	   $comenCentral = $this->getComenCentral($row);
	   foreach ($comenCentral as $central) {
           $row['QTFUNESTCPME'] = $central['QTFUNESTCPME'];
	       $row['QTDOAESTCPME'] = $central['QTDOAESTCPME'];
	       $row['QTCLIESTCPME'] = $central['QTCLIESTCPME'];
	       $row['QTCOMESTCPME'] = $central['QTCOMESTCPME'];
		   $this->updateComensal($row, null);	
        }
    }
	
	private function getComenCentral($row) {
	    // $sql = "SELECT QTFUNESTCPME, QTDOAESTCPME, QTCLIESTCPME, QTCOMESTCPME FROM (SELECT NVL(SUM(QTFUNESTCPME), 0) AS QTFUNESTCPME, NVL(SUM(QTDOAESTCPME), 0) AS QTDOAESTCPME, NVL(SUM(QTCLIESTCPME), 0) AS QTCLIESTCPME, NVL(SUM(QTCOMESTCPME), 0) AS QTCOMESTCPME FROM CARDIAPME C, PARAMPLANEJA P WHERE C.NRCARDPME    = :P_NRCARDPME    AND C.CDFILCENTPME = :P_CDFILCENTPME AND C.CDSERVFILI   = :P_CDSERVFILI   AND C.DTCARDIAPME  = :P_DTCARDIAPME  AND C.CDFILIAL    <> C.CDFILCENTPME AND C.CDFILIAL   =  P.CDFILIAL AND C.IDCARDESPEC  = 'N')";
	    // desconsidera a config paramplaneja, uma vez criado o cardapio já olha a paramplaneja
	    $sql = "SELECT QTFUNESTCPME, QTDOAESTCPME, QTCLIESTCPME, QTCOMESTCPME FROM (SELECT NVL(SUM(QTFUNESTCPME), 0) AS QTFUNESTCPME, NVL(SUM(QTDOAESTCPME), 0) AS QTDOAESTCPME, NVL(SUM(QTCLIESTCPME), 0) AS QTCLIESTCPME, NVL(SUM(QTCOMESTCPME), 0) AS QTCOMESTCPME FROM CARDIAPME C WHERE C.NRCARDPME    = :P_NRCARDPME    AND C.CDFILCENTPME = :P_CDFILCENTPME AND C.CDSERVFILI   = :P_CDSERVFILI   AND C.DTCARDIAPME  = :P_DTCARDIAPME  AND C.CDFILIAL    <> C.CDFILCENTPME AND C.IDCARDESPEC  = 'N')";
		
		$paramSelect = array(
            "P_CDFILCENTPME" => $row['CDFILCENTPME'],
            "P_NRCARDPME"    => $row['NRCARDPME'],
            "P_CDSERVFILI"   => $row['CDSERVFILI'],
            "P_DTCARDIAPME"  => $row['DTCARDIAPME']
        );
        
        return $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
	}
	
	public function getDataForm(DTO\Request\Row $request, DTO\Response $response) {
		$row = $request->getRow();
		
		// $this->updateComensal($row, $row['P_CDFILIAL']);	
		if ($row['P_CDFILCENTPME'] === $row['P_CDFILIAL']) {
	    	$this->updateComenCentral(array(
	    		'CDFILCENTPME' => $row['P_CDFILCENTPME'],
	    		'NRCARDPME' => $row['P_NRCARDPME'],
	    		'CDSERVFILI' => $row['P_CDSERVFILI'],
	    		'DTCARDIAPME' => $row['P_DTCARDIAPME'],
	    		'CDFILIAL' => $row['P_CDFILIAL'],
	    	));
		}
    	
		$sql = "SELECT CDFILCENTPME, NRCARDPME, NRCARDIAPME, CDSERVFILI, CDFILIAL, DTCARDIAPME, QTCOMESTCPME, QTCLIESTCPME, QTFUNESTCPME, QTDOAESTCPME, QTCOMEFECPME, QTCLIEFECPME, QTFUNEFECPME, QTDOAEFECPME, IDCARDESPEC FROM CARDIAPME WHERE CDFILCENTPME = :P_CDFILCENTPME AND NRCARDPME = :P_NRCARDPME AND CDSERVFILI = :P_CDSERVFILI AND CDFILIAL = :P_CDFILIAL AND DTCARDIAPME = :P_DTCARDIAPME AND IDCARDESPEC  = 'N' ORDER BY DTCARDIAPME";
		$paramSelect = array(
            "P_CDFILCENTPME" => $row['P_CDFILCENTPME'],
            "P_NRCARDPME"    => $row['P_NRCARDPME'],
            "P_CDSERVFILI"   => $row['P_CDSERVFILI'],
            "P_DTCARDIAPME"  => $row['P_DTCARDIAPME'],
            "P_CDFILIAL"     => ($row['P_CDFILIAL'] == NULL) ? $row['P_CDFILCENTPME'] : $row['P_CDFILIAL']
        );
        
        $dataForm = $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
        $response->addDataSet(new DataSet("dataForm", $dataForm));
	}
	
	public function updateComen(DTO\Request\Row $request, DTO\Response $response) {
		$row = $request->getRow();
		$filterData = $row['filterData'];
		$row['filterData']['CDFILIALINSERT'] = $row['filterData']['aggregated'] != null ? $row['filterData']['aggregated']['CDFILIAL'] : NULL;
		$this->connection->beginTransaction();
		try {
			$filial = ($filterData['aggregated'] == NULL) ? $filterData['cdfilcentpme'] : $filterData['aggregated']['CDFILIAL'];
	        $hasComen = $this->utilitiesCard->getCadiaPme($filterData['cdfilcentpme'], $filterData['NRCARDPME'], $filterData['service']['CDSERVFILI'], $row['DTCARDIAPME'], $filial);
			
	        if (count($hasComen) == 0) $this->elaboracaoCardapio->newCardia($row, 'N');
	         
	        $cardInDay = $this->elaboracaoCardapio->checkCardInDay($row);
	    	$aggregWithoutComen = $this->aggregatedWithoutComen($row, $filterData);
			foreach ($aggregWithoutComen as $aggregated) {
				if ($aggregated['CDFILIAL_CARD'] == null && ($filterData['aggregated'] == NULL || $filterData['aggregated'] === $aggregated['CDFILIAL'])) {
					$row['filterData']['CDFILIALINSERT'] = $aggregated['CDFILIAL'];
					$this->elaboracaoCardapio->newCardia($row, 'N');
				}
			}
			$result = count($hasComen) == 0 ? $cardInDay : $hasComen;
			$this->connection->commit();
			$response->addDataSet(new DataSet("hasComen", $result));
	    } catch (\Exception $e) {
			$this->connection->rollback();
	    	$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	private function aggregatedWithoutComen($row, $filterData) {
		$sql = "SELECT FT.CDFILCENTPME, FT.CDFILIAL, FT.CDSERVFILI, C.CDFILIAL AS CDFILIAL_CARD FROM (SELECT FILITIPOCARD.CDFILCENTPME, FILITIPOCARD.CDFILIAL, SERVTIPOCARD.CDSERVFILI FROM FILITIPOCARD, SERVTIPOCARD, PARAMPLANEJA WHERE (FILITIPOCARD.CDFILCENTPME = :P_CDFILCENTPME) AND (FILITIPOCARD.CDTIPOCARD = :P_CDTIPOCARD) AND (SERVTIPOCARD.CDFILCENTPME = FILITIPOCARD.CDFILCENTPME) AND (SERVTIPOCARD.CDTIPOCARD = FILITIPOCARD.CDTIPOCARD) AND (SERVTIPOCARD.CDSERVFILI = :P_CDSERVFILI) AND (FILITIPOCARD.CDFILIAL = PARAMPLANEJA.CDFILIAL) AND (FILITIPOCARD.IDTCARATFILI = 'S')) FT LEFT JOIN CARDIAPME C ON C.CDFILCENTPME = FT.CDFILCENTPME AND C.CDSERVFILI = FT.CDSERVFILI AND C.CDFILIAL = FT.CDFILIAL AND C.NRCARDPME = :P_NRCARDPME AND C.DTCARDIAPME = :P_DTCARDIAPME";
		$paramSelect = array(
            "P_CDFILCENTPME" => $filterData['cdfilcentpme'],
            "P_CDTIPOCARD"   => $filterData['typeMenu']['CDTIPOCARD'],
            "P_NRCARDPME"    => $filterData['NRCARDPME'],
            "P_CDSERVFILI"   => $filterData['service']['CDSERVFILI'],
            "P_DTCARDIAPME"  => $row['DTCARDIAPME']
        );
        
        return $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
	}
}