import {
  Loader, Metadata, IEventParam, IDictionary, Config, I18n,
} from '@zeedhi/core';
import {
  Form, Select, Button, CheckboxMultiple,
} from '@zeedhi/common';
import { PmeMask } from '@/config';

export class Pme01100CustoPlanejadoRealizadoController {
  public reportController = Loader.getInstance('ReportController');

  public appController = Loader.getInstance('AppController');

  public utilities = Loader.getInstance('Utilities');
  
  public modalController = Loader.getInstance('ModalController');

  public defaultReport({ component }: IEventParam<Select>) {
    component.setValue(component.datasource.data[0]);
  }
  
  private impTipoProduto = {
    ESTOCAVEIS: 'S',
    NAOESTOCAVEIS: 'S',
  };
  
  public setFilterTpCardapio({ component }: IEventParam<Select>) {
    const filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'NRPASTATCARD',
        operator: '=',
        value: null,
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public setFilterAgregAgrup({ component }: IEventParam<Select>) {
    const agrupUnid = Metadata.getInstance<Select>('GRUPFILI');
    
    const conditions = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDGRUPFILI',
        operator: '=',
        value: agrupUnid.value?.CDGRUPFILI ? agrupUnid.value.CDGRUPFILI : 'T',
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  }
  
  private setParams(row: IDictionary) {
    const mask: string = (Config.masks.productMask as PmeMask).pattern;
    row.CDSERVFILI = row.SERVICO ? row.SERVICO : null;
    row.CDFILIAL = row.UNIDADE ? row.UNIDADE : null;
    row.CDFILCENTPME = this.appController.getCdCentral();
    row.NMFILCENTPME = this.appController.getNmCentral();
    row.CDGRUPFILI = row.GRUPFILI ? row.GRUPFILI.CDGRUPFILI : null;
    row.NMGRUPFILI = row.GRUPFILI ? row.GRUPFILI.NMGRUPFILI : I18n.translate('Nenhum');
    row.NIVEL = row.NIVEL !== undefined ? row.NIVEL : 0;
    row.TAMACNIVEL = this.utilities.calcTamacNivel(row.NIVEL, 'produto');
    row.ESTOCAVEIS = this.impTipoProduto.ESTOCAVEIS;
    row.NAOESTOCAVEIS = this.impTipoProduto.NAOESTOCAVEIS;
    row.REPORT_NAME = row.REPORT.REPORT_NAME;
    row.CD_REPORT_CODE = row.REPORT.CD_REPORT_CODE;
    row.AGRUPMASK = this.utilities.getReportMaskNivel(mask, row.NIVEL);
    row.DTMESINIC = row.DTDE;
    row.DTMESFINA = row.DTATE;
    row.CDTIPOCUSTOREAL = row.TIPOCUSTOREAL.CDTIPOCUSTO;
    row.NMTIPOCUSTOREAL = row.TIPOCUSTOREAL.NMTIPOCUSTO;
    row.CDTIPOCUSTOPLAN = row.TIPOCUSTOPLAN.CDTIPOCUSTO;
    row.NMTIPOCUSTOPLAN = row.TIPOCUSTOPLAN.NMTIPOCUSTO;
  }
  
  public generateReport({ component }: IEventParam<Button>) {
    const form = Metadata.getInstance<Form>('FORMCUSTOPLAREAL');
    const tipoProduto = Metadata.getInstance<CheckboxMultiple>('TIPOPRODUTO');

    if (this.utilities.centralIsValid() && this.reportController.formIsValid(form)) {
      const row: IDictionary = form.value;
      
      this.setParams(row);
      if (row.CDFILCENTPME == null) this.utilities.showAlert('alertError', 'Central não informada.', 'error');
      else if (tipoProduto.value.length === 0) {
        this.utilities.showAlert('alertError', 'Tipo de Produto deve ser informado.', 'error');
      
      /* const dtFinMax = moment(row.DTDE, 'DD/MM/YYYY').add(15, 'd');
      if ((row.TIPOCARD.length === 0 && row.CDGRUPFILI == null
          && row.CDFILIAL.length === 0 && row.GRUPOSERVICO == null && row.SERVICO.length === 0)
          || (row.DTDE !== row.DTATE && moment(row.DTATE, 'DD/MM/YYYY') > dtFinMax)
      ) {
        DialogService.show({
          name: 'confirmReport',
          type: 'warning',
          title: 'Atenção',
          text: 'Caso não seja definido opções de filtro ou informar um período superior a 15 dias o '+
          'processamento pode demorar. Deseja continuar?',
          buttons: [
            {
              name: 'yes',
              component: 'ZdButton',
              label: 'Sim',
              events: { click: () => this.reportController.callGenerateReport(component, row, 'report/custoPlanRealCard') },
              outline: true,
            },
            {
              name: 'no',
              component: 'ZdButton',
              label: 'Não',
              events: { click: () => DialogService.hide() },
            },
            {
              name: 'progressLine',
              component: 'ZdProgress',
              label: 'Progress',
              indeterminate: true,
              cssClass: 'loading-progress-line',
            },
          ],
        });
      } */
      } else {
        this.reportController.callGenerateReport(component, row, 'report/custoPlanRealCard');
      }
    }
  }
  
  public setTipoProduto({ component }: IEventParam<CheckboxMultiple>) {
    if (component.value) {
      const items = component.value.join('_');
      if (items.includes('ESTOCAVEIS')) this.impTipoProduto.ESTOCAVEIS = 'S';
      else this.impTipoProduto.ESTOCAVEIS = 'N';

      if (items.includes('NAOESTOC')) this.impTipoProduto.NAOESTOCAVEIS = 'S';
      else this.impTipoProduto.NAOESTOCAVEIS = 'N';
    }
  }
  
  public disabledFocus({ component }: IEventParam<Select>) {
    component.datasource.loading = false;
  }
  
  public setFilterServAgrupUnid({ component }: IEventParam<Select>) {
    let agrupUnidValue;
    let grupServValue;
    
    component.datasource.loading = true;
    const unidade = Metadata.getInstance<Select>('UNIDADE');
    const unidadeValue = (unidade.value.length !== 0) ? unidade.value.join('_') : 'T';
    
    const grupserv = Metadata.getInstance<Select>('GRUPOSERVICO');
    grupServValue = grupserv.value ? grupserv.value : 'T';
    
    const agrupUnid = Metadata.getInstance<Select>('GRUPFILI');
    agrupUnidValue = agrupUnid.value ? agrupUnid.value : 'T';
    
    const cdfilcentpme = this.appController.getCdCentral();
    
    if (typeof (agrupUnidValue) === 'object') {
      agrupUnidValue = agrupUnid.value.CDGRUPFILI;
    }
    
    if (typeof (grupServValue) === 'object') {
      grupServValue = grupserv.value.CDSERVFILI;
    }
    
    const filter = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: unidadeValue,
      },
      {
        name: 'CDGRUPSERV',
        operator: '=',
        value: grupServValue,
      },
      {
        name: 'CDGRUPFILI',
        operator: '=',
        value: agrupUnidValue,
      },
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: cdfilcentpme,
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
}
