import {
  Datasource, Http, IDictionary, Loader, Metadata, IEventParam, Config, I18n, dayjs as moment,
  DatasourceFactory,
} from '@zeedhi/core';
import {
  Button, Date as ZdDate, Form, GridEditable, Select, TextInput, CheckboxMultiple,
  DialogService, Grid, Modal, ModalService, IButton, Tabs, LoadingService,
} from '@zeedhi/common';
import { PmeMask } from '@/config';

export class Pme02000RetiradaInventarioController {
  private modalAutoRemoval!: Modal;

  private modalProdIrreg!: Modal;
  
  private modalProdIrregRetAuto!: Modal;
  
  private modalProdNaoPlanejado!: Modal;
  
  private modalRomaneio!: Modal;

  private loadingAutoRemoval = false;

  private nrlistaProd = '';

  private titleProdIrreg = '';

  public appController = Loader.getInstance('AppController');
  
  public modalController = Loader.getInstance('ModalController');
  
  public utilities = Loader.getInstance('Utilities');
  
  public gridEditController = Loader.getInstance('GridEditableController');

  public paramsAutoRemoval: IDictionary = {};
  
  public allItemsRomaneio: IDictionary = [];
  
  private get titleProdIrregRetAuto() {
    return `${I18n.translate('Produtos Irregulares')}`;
  }

  public async createModal() {
    LoadingService.show(' ');
    this.modalRomaneio = await ModalService.createFromJsonPost('',
      { jsonPath: 'pme/medicao/modals/formRomaneio' });
    // this.modalController.createModal('pme/medicao/modals/formRomaneio');
    this.modalAutoRemoval = await ModalService.createFromJsonPost('', { jsonPath: 'pme/medicao/modals/retiradaAutomatica' });
    this.modalProdIrreg = await ModalService.createFromJsonPost('',
      { jsonPath: 'pme/medicao/modals/gridProdIrreg' });
    this.modalProdIrregRetAuto = await ModalService.createFromJsonPost('',
      { jsonPath: 'pme/medicao/modals/gridProdIrregRetAuto' });
    this.modalProdNaoPlanejado = await ModalService.createFromJsonPost('',
      { jsonPath: 'pme/medicao/modals/gridProdNaoPlanejado' });
    LoadingService.hide();
  }

  public destroyModal() {
    this.modalController.destroyModal();
    if (this.modalRomaneio) this.modalRomaneio.destroy();
    if (this.modalAutoRemoval) this.modalAutoRemoval.destroy();
    if (this.modalProdIrreg) this.modalProdIrreg.destroy();
    if (this.modalProdIrregRetAuto) this.modalProdIrregRetAuto.destroy();
    if (this.modalProdNaoPlanejado) this.modalProdNaoPlanejado.destroy();
  }

  public openModalAutoRemoval() {
    const grid: GridEditable = Metadata.getInstance('romaneioItemsGrid');
    const cloneConfig = grid.datasource.clone();
    const cloneDs = DatasourceFactory.factory(cloneConfig);
    cloneDs.loadAll = true;
    LoadingService.show();
    cloneDs.get().then(() => {
      const items = cloneDs.data;
      this.allItemsRomaneio = items;
      const some = items.some((item: IDictionary) => item.IDCNTRESTOQ === 'N' || item.IDPRODATIV === 'I');
      if (some) {
        const text = 'Operação bloqueada! Há itens com produtos que não controlam estoque'
        + ' ou que não estão cadastrados para unidade ou inativos.';
        DialogService.show({
          name: 'errorItemsRomaneio',
          type: 'warning',
          title: 'Atenção',
          text,
          buttons: [
            {
              name: 'ok',
              component: 'ZdButton',
              label: 'OK',
              events: { click: () => DialogService.hide() },
            },
          ],
        });
      } else this.modalAutoRemoval.show();
    }).finally(() => {
      LoadingService.hide();
    });
  }

  public closeModalAutoRemoval() {
    if (!this.loadingAutoRemoval) this.modalAutoRemoval.hide();
  }

  public openModalProdIrreg() {
    this.modalProdIrreg.show();
  }
  
  public openModalProdIrregRetAuto() {
    this.modalProdIrregRetAuto.show();
  }
  
  public openModalProdNaoPlanejado() {
    this.modalProdNaoPlanejado.show();
  }
  
  public closeModalProdNaoPlanejado() {
    this.modalProdNaoPlanejado.hide();
  }

  public closeModalProdIrreg() {
    const gridRomaneio: GridEditable = Metadata.getInstance('romaneioGrid');
    gridRomaneio.datasource.get();
    this.modalProdIrreg.hide();
    this.closeModalAutoRemoval();
  }
  
  public closeModalProdIrregRetAuto() {
    const gridRomaneio: GridEditable = Metadata.getInstance('romaneioGrid');
    this.modalProdIrregRetAuto.hide();
    this.closeModalAutoRemoval();
    gridRomaneio.datasource.get();
  }

  public dtIni = '';

  public dtFin = '';

  private situation = {
    PENDENTE: 'S',
    REALIZADO: 'N',
  };

  private currentRow: IDictionary = {};
  
  public disableFields = false;
  
  public disableGridAdd = false;

  public showCost = 'N';
  
  public showItems = false;
  
  public itemSaved = false;

  public colsRomaneio = 12;

  public selectedItems: IDictionary = [];
  
  private paramfilial: IDictionary = {};
  
  private paramplaneja: IDictionary = {};

  public prodEditable({ row }: IDictionary) {
    return row.__is_new;
  }

  public get visibilityItems() {
    const gridRomaneio: GridEditable = Metadata.getInstance('romaneioGrid');
    // if (gridRomaneio.datasource.currentRow.DTINVENTRETI) this.colsRomaneio = 12;
    // else this.colsRomaneio = 12;
    return !!gridRomaneio.datasource.currentRow.DTINVENTRETI;
  }
  
  public getConfigRetirada() {
    this.disableFields = true;
    const central = this.appController.getCdCentral();
    if (central) {
      Http.post('/configRetirada', { CDCENTRAL: central }).then((response) => {
        const result = response.data.dataset.result;
        const config = result.paramplaneja;
        this.paramplaneja = result.paramplaneja;
        this.paramfilial = result.paramfilial;
        if (config[0] && config[0].IDUTRETBASEINVE === 'S') {
          this.disableFields = false;
          this.disableGridAdd = false;
          this.utilities.getProcessDate(central).then((date: string) => {
            this.dtIni = date;
            this.dtFin = date;
          });
        } else {
          this.disableFields = true;
          this.disableGridAdd = true;
          this.utilities.showAlert('erroConfig',
            'Central não está habilitada para realizar a retirada do merenda com base no inventário.',
            'error');
        }
      });
    }
  }

  public setSituation({ component }: IEventParam<CheckboxMultiple>) {
    if (component.value) {
      const items = Array.isArray(component.value) ? component.value.join('_') : component.value;

      if (items.includes('PENDENTE')) this.situation.PENDENTE = 'S';
      else this.situation.PENDENTE = 'N';
      
      if (items.includes('REALIZADO')) this.situation.REALIZADO = 'S';
      else this.situation.REALIZADO = 'N';

      this.setFilterRomaneio(component);
    }
  }

  public setCost({ component }: IEventParam<CheckboxMultiple>) {
    if (component.value) {
      const items = Array.isArray(component.value) ? component.value.join('_') : component.value;
      this.showCost = items.includes('SHOWCUSTO') ? 'S' : 'N';
      this.reloadItems();
    }
  }
  
  public validateAddRomaneio() {
    const agregada: Select = Metadata.getInstance('UNIDADE');
    Http.post('/configRetirada', { CDCENTRAL: agregada.value }).then((response) => {
      this.paramplaneja = response.data.dataset.result.paramplaneja;
      this.paramfilial = response.data.dataset.result.paramfilial;
      if (response.data.dataset.result.paramplaneja[0].IDPRINVENRETI === 'S'
        && this.disableFields === false) this.disableGridAdd = false;
      else this.disableGridAdd = true;
    });
  }

  public setFilterRomaneio({ component }: IDictionary) {
    if (component.name === 'UNIDADE') this.validateAddRomaneio();
    const grid: GridEditable = Metadata.getInstance('romaneioGrid');
    const agregada: Select = Metadata.getInstance('UNIDADE');
    const formFiltro: Form = Metadata.getInstance('formFiltro');
    
    if (formFiltro.validate()) {
      if (this.utilities.dateIsValid(this.dtIni, this.dtFin)) {
        let efetivados = null;
        if (this.situation.PENDENTE === 'S' && this.situation.REALIZADO === 'S') efetivados = 'T';
        else if (this.situation.PENDENTE === 'S') efetivados = 'N';
        else if (this.situation.REALIZADO === 'S') efetivados = 'S';
    
        const conditions = [
          {
            name: 'P_CDFILIAL',
            operator: '=',
            value: agregada.value,
          },
          {
            name: 'P_DTINICIAL',
            operator: '=',
            value: this.dtIni,
          },
          {
            name: 'P_DTFINAL',
            operator: '=',
            value: this.dtFin,
          },
          {
            name: 'P_IDRETIEFET',
            operator: '=',
            value: efetivados,
          },
        ];
    
        grid.datasource.filter = conditions;
        grid.datasource.get();
      } else {
        this.utilities.showAlert('alertError', 'pme::dataIniMaior', 'error');
        grid.datasource.filter = [];
        grid.datasource.data = [];
      }
    } else {
      grid.datasource.filter = [];
      grid.datasource.data = [];
    }
  }

  public setFilterItems({ component }: IEventParam<Datasource>) {
    this.currentRow = component.currentRow;

    const addReti = Metadata.getInstance<Button>('addButtonReti');
    const addItem = Metadata.getInstance<Button>('addButtonItem');
    
    addReti.disabled = this.disableGridAdd;
    addItem.disabled = this.disableGridAdd;
    
    this.reloadItems();
  }

  public reloadItems() {
    const grid: GridEditable = Metadata.getInstance('romaneioItemsGrid');
    const agregada: Select = Metadata.getInstance('UNIDADE');
    const central = this.appController.getCdCentral();
    // const dtInvent = moment(this.currentRow.DTINVENTRETI, 'DD/MM/YYYY').add(1, 'd').format('DD/MM/YYYY');
    const dtInvent = moment(this.currentRow.DTINVENTRETI, 'DD/MM/YYYY').add(1, 'd').format('DD/MM/YYYY');
    // if (this.paramFilial && (this.paramFilial.IDORDAJUINVE === null ||
    // this.paramFilial.IDORDAJUINVE === 'P')) date =
    // moment(dtInvent, 'DD/MM/YYYY').add(1, 'd').format('DD/MM/YYYY');
    // else date = dtInvent;
    
    if (this.currentRow.NRINVENTRETI) {
      const conditions = [
        {
          name: 'P_CDFILIAL',
          operator: '=',
          value: agregada.value,
        },
        {
          name: 'P_IDEXIBCUS',
          operator: '=',
          value: this.showCost,
        },
        {
          name: 'P_DTPOSIESTQ',
          operator: '=',
          value: dtInvent,
        },
        {
          name: 'P_CDFILCENTPME',
          operator: '=',
          value: central,
        },
        {
          name: 'P_NRINVENTRETI',
          operator: '=',
          value: this.currentRow.NRINVENTRETI,
        },
      ];
  
      grid.datasource.filter = conditions;
      // grid.datasource.get()
      this.gridEditController.defaultState(grid).then(() => {
        document.getElementsByClassName('gridRomaneioItems')[0].scrollIntoView();
      });
    }
  }

  public getFilterProdItem() {
    const agregada: Select = Metadata.getInstance('UNIDADE');
    return [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: agregada.value,
      },
      {
        name: 'NRINVENTRETI',
        operator: '=',
        value: this.currentRow.NRINVENTRETI ? this.currentRow.NRINVENTRETI : null,
      },
    ];
  }
  
  public setFilterService() {
    const conditions = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: this.paramsAutoRemoval.CDFILIAL ? this.paramsAutoRemoval.CDFILIAL : null,
      },
      {
        name: 'IDTIPOSERV',
        operator: '=',
        value: this.utilities.isMerenda() ? '' : 'N',
      },
      {
        name: 'IDSERVATIV',
        operator: '=',
        value: 'S',
      },
    ];
    return conditions;
  }

  public setFilterProdForm({ component }: IEventParam<Select>) {
    const agregada: Select = Metadata.getInstance('UNIDADE');
    const conditions = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: agregada.value,
      },
      {
        name: 'NRINVENTRETI',
        operator: '=',
        value: this.romaneio.NRINVENTRETI ? this.romaneio.NRINVENTRETI : null,
      },
    ];
    component.datasource.filter = conditions;
    component.datasource.get();
  }

  public saveItems({ component }: IDictionary) {
    const formFiltro = Metadata.getInstance<Form>('formFiltro');
    if (this.currentRow.IDRETIEFET === 'S') {
      this.utilities.showAlert(
        'errorEdicaoItem',
        'Operação bloqueada. Romaneio já efetivado.',
        'error',
      );
    } else if (formFiltro.validate()) {
      const grid = this.utilities.returnGrid(component) as GridEditable;
      const rows: IDictionary = grid.getEditedRows(false, true);

      if (grid.isGridValid(true) && rows.length > 0) {
        rows.forEach((e: IDictionary) => {
          e.CDFILCENTPME = this.appController.getCdCentral();
          e.CDFILIAL = formFiltro.value.UNIDADE;
          e.DTINVENTRETI = this.currentRow.DTINVENTRETI;
          e.NRINVENTRETI = this.currentRow.NRINVENTRETI;
        });
        this.callSave('/saveItemsRomaneio', rows, grid);
      }
    }
  }

  public deleteItems({ component }: IEventParam<Button>) {
    const grid = this.utilities.returnGrid(component) as GridEditable;
    this.selectedItems = grid.selectedRows;
    if (this.currentRow.IDRETIEFET === 'N') {
      if (this.selectedItems.length === grid.datasource.total) {
        this.selectedItems = grid.selectedRows;
        DialogService.show({
          name: `${component.name}Lancamento`,
          type: 'warning',
          title: 'Atenção',
          text: 'Ao excluir o único item do lançamento, o mesmo também será excluído. Deseja continuar?',
          buttons: [
            {
              name: 'yes',
              component: 'ZdButton',
              label: 'Sim',
              events: { click: () => this.callDeleteItems() },
              outline: true,
            },
            {
              name: 'no',
              component: 'ZdButton',
              label: 'Não',
              events: { click: () => DialogService.hide() },
            },
          ],
        });
      } else this.gridEditController.deleteCheckedRows({ component });
    } else this.utilities.showAlert('errorExclusaoItem', 'Operação bloqueada. Romaneio já efetivado.', 'error');
  }

  public callDeleteItems() {
    const gridItem: GridEditable = Metadata.getInstance('romaneioItemsGrid');
    const gridRomaneio: GridEditable = Metadata.getInstance('romaneioGrid');
    const agregada: Select = Metadata.getInstance('UNIDADE');
    this.selectedItems[0].CDFILIAL = agregada.value;

    Http.post('/deleteItemRomaneio', { requestType: 'Row', row: this.selectedItems }).then(() => {
      gridItem.cancelEditedRows();
      gridItem.datasource.get();
      gridItem.selectedRows = [];
      this.selectedItems = [];
      gridRomaneio.datasource.get();
      DialogService.hide();
      this.utilities.showAlert('alert', 'Excluído com sucesso!', 'success');
    }).catch((error: string) => {
      DialogService.hide();
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
    });
  }

  public deleteRomaneio({ component }: IEventParam<Button>) {
    const gridRomaneio: GridEditable = Metadata.getInstance('romaneioGrid');
    const checkedRows = gridRomaneio.selectedRows;
    if (checkedRows.length === 0) this.utilities.showAlert('alert', 'Nenhum registro selecionado.', 'warning');
    else if (!this.romaneioIsValid(checkedRows)) {
      this.utilities.showAlert(
        'alert',
        'Operação bloqueada. Romaneio já efetivado.',
        'warning',
      );
    } else {
      DialogService.show({
        name: `${component.name}`,
        type: 'warning',
        title: 'Atenção',
        text: 'Ao excluir o lançamento, todos os seus itens serão excluídos. Deseja continuar?',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'Sim',
            events: { click: () => this.callDeleteRomaneio() },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'Não',
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    }
  }

  public romaneioIsValid(rows: IDictionary): boolean {
    let valid = true;
    rows.forEach((e: IDictionary) => {
      if (e.IDRETIEFET === 'S') valid = false;
    });
    return valid;
  }

  public callDeleteRomaneio() {
    const gridItem: GridEditable = Metadata.getInstance('romaneioItemsGrid');
    const gridRomaneio: GridEditable = Metadata.getInstance('romaneioGrid');
    const rows = gridRomaneio.selectedRows;

    Http.post('/deleteRomaneio', { requestType: 'Row', row: rows }).then(() => {
      gridRomaneio.cancelEditedRows();
      gridRomaneio.datasource.get();
      gridRomaneio.selectedRows = [];
      this.selectedItems = [];
      gridItem.datasource.get();
      DialogService.hide();
      this.utilities.showAlert('alert', 'Excluído com sucesso!', 'success');
    }).catch((error: string) => {
      DialogService.hide();
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
    });
  }

  public itemIsValid({ row }: IDictionary) {
    let color = '';
    if (this.currentRow.IDRETIEFET === 'N' && row.QTESTOQDIA < row.QTITINVENTRETI) color = 'red';
    return color;
  }

  public cssEfetivado({ row }: IDictionary) {
    let colorClass = 'green-efetivos';
    if (row.IDRETIEFET === 'N') colorClass = 'red-efetivos';
    return colorClass;
  }

  public produtoComponentProps = {
    component: 'ZdSelect',
    dataText: [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ],
    formatterDataText: [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ],
    dataValue: 'CDPRODUTO',
    dataValueOut: [
      {
        column: 'CDARVPROD',
        columnOnGrid: 'CDARVPROD',
      },
      {
        column: 'NMPRODUTO',
        columnOnGrid: 'NMPRODUTO',
      },
      {
        column: 'SGUNIDADE',
        columnOnGrid: 'SGUNIDADE',
      },
    ],
    dense: true,
    datasource: {
      uniqueKey: 'CDPRODUTO',
      route: '/find/prodUltimoNivelItem',
      type: 'rest',
      limit: 10,
      lazyLoad: true,
    },
    validations: {
      required: {
        message: 'Campo obrigatório',
      },
    },
  };

  private callSave(url: string, rows: IDictionary, grid: GridEditable) {
    if (grid) grid.datasource.loading = true;
    Http.post(url, { requestType: 'Row', row: rows }).then((response) => {
      if (response) {
        this.utilities.showAlert('alertSuccess', 'Produto incluído na retirada.', 'green');
        this.gridEditController.defaultState(grid);
      }
    }).catch((error: string) => {
      this.utilities.showAlert('alertError', `pme::${error}`, 'error');
    }).finally(() => {
      if (grid) grid.datasource.loading = false;
    });
  }

  get maskProd() {
    return [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ];
  }

  // modalRomaneio functions
  private romaneio: IDictionary = {};

  public showModalItems = false;

  public get notShowModalItems() {
    return !this.showModalItems;
  }

  public addRomaneio() {
    const agregada: Select = Metadata.getInstance('UNIDADE');
    if (agregada.value) {
      if (!this.modalRomaneio) {
        this.createModal().then(() => {
          this.showModalItems = false;
          this.modalRomaneio.show();
        });
      } else {
        this.showModalItems = false;
        this.modalRomaneio.show();
      }
      // this.focusOnData();
    } else this.utilities.showAlert('errorUnidade', 'Unidade não selecionada.', 'error');
  }

  public closeRomaneio(closeDialog = false) {
    const gridRomaneio: GridEditable = Metadata.getInstance('romaneioGrid');
    const gridItems: GridEditable = Metadata.getInstance('romaneioItemsGrid');
    
    this.romaneio = {};
    this.itemSaved = false;
    if (closeDialog) DialogService.hide();
    this.modalRomaneio.hide();
    this.modalRomaneio.destroy();
    this.modalRomaneio = this.utilities.modalEmpty;
    gridRomaneio.datasource.get();
    gridItems.datasource.get();
  }

  public focusOnData({ component }: IEventParam<ZdDate>) {
    component.value = this.utilities.dtProcessamento;
    setTimeout(() => {
      this.utilities.focusOnField('DATEROMANEIO');
    }, 1);
  }

  public confirmaRomaneio() {
    const formRomaneio = Metadata.getInstance<Form>('formRomaneio');
    if (formRomaneio.validate()) {
      const agregada: Select = Metadata.getInstance('UNIDADE');
      this.romaneio = {
        CDFILIAL: agregada.value,
        DTINVENTRETI: formRomaneio.value.DATEROMANEIO,
        DSINVENTRETI: formRomaneio.value.DESCRIPTION,
        IDRETIEFET: 'N',
        NRINVENTRETI: null,
      };
      this.showModalItems = true;
      setTimeout(() => {
        this.utilities.focusOnField('PRODUCT');
      }, 1);
    }
  }

  public saveItemForm() {
    const formItems = Metadata.getInstance<Form>('formItems');
    const product = Metadata.getInstance<Select>('PRODUCT');
    const qtd = Metadata.getInstance<TextInput>('QTD');
    
    if (formItems.validate()) {
      const row = {
        romaneio: this.romaneio,
        item: {
          CDFILIAL: this.romaneio.CDFILIAL,
          CDPRODUTO: formItems.value.PRODUCT,
          QTITINVENTRETI: formItems.value.QTD,
          NRINVENTRETI: this.romaneio.NRINVENTRETI,
        },
      };
      
      Http.post('/saveRomaneio', row).then((response) => {
        this.itemSaved = true;
        this.romaneio.NRINVENTRETI = response.data.dataset.nrseqinvent[0];
        this.utilities.showAlert('alert', 'Produto incluído na retirada.', 'success');
      }).catch((error: string) => {
        this.utilities.showAlert('error', error, 'error');
      });

      product.setValue('');
      qtd.value = '';
      this.utilities.focusOnField('PRODUCT');
    }
  }

  public confirmFormRomaneio() {
    const product = Metadata.getInstance<Select>('PRODUCT');
    const qtd = Metadata.getInstance<TextInput>('QTD');
    
    if (product.value && qtd.value) {
      const row = {
        romaneio: this.romaneio,
        item: {
          CDFILIAL: this.romaneio.CDFILIAL,
          CDPRODUTO: product.value,
          QTITINVENTRETI: qtd.value,
          NRINVENTRETI: this.romaneio.NRINVENTRETI,
        },
      };
      LoadingService.show(' ');
      Http.post('/saveRomaneio', row).then(() => {
        this.utilities.showAlert('alert', 'Salvo com sucesso!', 'success');
        this.closeRomaneio(false);
      }).finally(() => {
        LoadingService.hide();
      });
    } else if (!this.itemSaved) {
      DialogService.show({
        name: 'finalizarRomaneio',
        type: 'warning',
        title: 'Atenção',
        text: 'Nenhum item foi incluso no romaneio. Deseja cancelar a inclusão?',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'Sim',
            events: { click: () => this.closeRomaneio(true) },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'Não',
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    } else this.closeRomaneio(false);
  }

  /**
   * Rotina de Retirada Automática
   */
  private activatedButton: IButton | undefined;

  public get disableAutoRemoval() {
    return this.currentRow.IDRETIEFET !== 'N';
  }

  // gravar
  public recordStock({ component }: IEventParam<Button>) {
    this.activatedButton = component;
    this.currentRow.ZERAR = false;
    // if (this.stockIsValid()) this.validateAutoRemoval(this.currentRow);
    this.validateAutoRemoval(this.currentRow);
  }

  // zerar
  public resetStock({ component }: IEventParam<Button>) {
    this.activatedButton = component;
    this.currentRow.ZERAR = true;
    // if (this.stockIsValid()) this.validateAutoRemoval(this.currentRow);
    this.validateAutoRemoval(this.currentRow);
  }

  private validateAutoRemoval(row: IDictionary) {
    LoadingService.show(' ');
    if (this.paramsAutoRemoval.DTINVENTRETI) this.paramsAutoRemoval.DTVALESTOQ = this.paramsAutoRemoval.DTINVENTRETI;
    Http.post('/validateAutoRemoval', row).then((response) => {
      this.paramsAutoRemoval = response.data.dataset.params[0];
      const dtIni = moment(this.paramsAutoRemoval.DTINICIAL, 'DD/MM/YYYY');
      let dtInvent = moment(this.paramsAutoRemoval.DTINVENTRETI, 'DD/MM/YYYY');
      let msg = '';
      if (dtInvent.diff(dtIni, 'day') >= 60) {
        dtInvent = dtInvent.subtract(60, 'day');
        this.paramsAutoRemoval.DTINICIAL = dtInvent.format('DD/MM/YYYY');
      
        msg = 'Data do último inventário superior a sessenta dias. '
        + 'Deseja considerar o período de sessenta dias para que as retiradas possam ser geradas?';
        this.dialogDtIni(msg, '60days');
        LoadingService.hide();
      } else if (dtInvent.diff(dtIni, 'day') >= 30) {
        dtInvent = dtInvent.subtract(30, 'day');
        this.paramsAutoRemoval.DTINICIAL = dtInvent.format('DD/MM/YYYY');
      
        msg = 'Data do último inventário maior que trinta dias. Deseja continuar?';
        this.dialogDtIni(msg, '30days');
        LoadingService.hide();
      } else this.callAutoRemoval();
    }).catch((error: string) => {
      const arrError = error.split('_');
      const msgError = arrError[0];
      
      if (msgError === 'Produtos que não controlam estoque') {
        this.titleProdIrreg = 'Produtos que não controlam estoque';
        this.utilities.showAlert('error', 'Operação bloqueada! Existem produtos que não controlam estoque.', 'error');
      } else if (msgError === 'Operação bloqueada! Produto não cadastrado para unidade ou inativo.') {
        this.titleProdIrreg = 'Produtos Inativos';
        this.utilities.showAlert('error', 'Operação bloqueada! Produto não cadastrado para unidade ou inativo.', 'error');
        this.nrlistaProd = arrError[1];
        this.openModalProdIrreg();
      } else this.utilities.showAlert('error', msgError, 'error');
      /* if (msgError.includes('pme')) {
        this.nrlistaProd = arrError[1];
        this.openModalProdIrreg();
      } */
      LoadingService.hide();
    });
  }

  private dialogDtIni(text = '', name = 'dialog') {
    DialogService.show({
      name,
      type: 'warning',
      title: 'Data do Último Inventário',
      text,
      buttons: [
        {
          name: 'sim',
          component: 'ZdButton',
          label: 'Sim',
          events: {
            click: () => {
              DialogService.hide();
              this.callAutoRemoval();
            },
          },
          outline: true,
        },
        {
          name: 'nao',
          component: 'ZdButton',
          label: 'Não',
          events: { click: () => DialogService.hide() },
        },
      ],
    });
  }

  private dialogEfet() {
    DialogService.show({
      name: `dialogEfet${this.paramsAutoRemoval.CDFILIAL + this.paramsAutoRemoval.NRINVENTRETI}`,
      type: 'warning',
      title: 'Efetivos no Período',
      text: 'Não há lançamento de efetivos no período. Será realizado apenas o ajuste de inventário. Deseja continuar?',
      buttons: [
        {
          name: 'sim',
          component: 'ZdButton',
          label: 'Sim',
          events: {
            click: () => {
              this.paramsAutoRemoval.ERROR = null;
              this.paramsAutoRemoval.IDGERARETI = 'N';
              this.callAutoRemoval();
              DialogService.hide();
            },
          },
          outline: true,
        },
        {
          name: 'nao',
          component: 'ZdButton',
          label: 'Não',
          events: {
            click: () => {
              this.deleteConsplan(this.paramsAutoRemoval);
              DialogService.hide();
            },
          },
        },
      ],
    });
  }
  
  private dialogUnplannedProd() {
    DialogService.show({
      name: 'infoProdNaoPlan',
      type: 'info',
      title: 'Produtos Não Planejados',
      text: 'Existem produtos não planejados na retirada automática, os mesmos '
      + 'serão lançados no serviço padrão parametrizado para retirada automática ou '
      + 'podem ter o serviço definido para cada produto. Deseja continuar ou informar serviço específico?',
      persistent: false,
      // text: `${I18n.translate('Os produtos não planejados serão lançados no '+
      // 'serviço parametrizado para retirada automática. Deseja continuar?',
      // { PARAM1: servRet.CDSERVRETPME, PARAM2: servRet.NMSERVICO })}`,
      buttons: [
        {
          name: 'yes',
          component: 'ZdButton',
          label: 'Sim',
          events: {
            click: () => {
              this.paramsAutoRemoval.ERROR = null;
              this.paramsAutoRemoval.checkUnplannedProd = false;
              this.callAutoRemoval();
              DialogService.hide();
            },
          },
          outline: true,
        },
        {
          name: 'no',
          component: 'ZdButton',
          label: 'Informar serviços',
          events: {
            click: () => {
              this.openModalProdNaoPlanejado();
              DialogService.hide();
            },
          },
        },
        {
          name: 'cancelRet',
          component: 'ZdButton',
          label: 'Cancelar',
          events: {
            click: () => {
              DialogService.hide();
            },
          },
        },
      ],
    });
  }

  private stockIsValid() {
    let valid = true;
    let text = '';
    const gridItems: GridEditable = Metadata.getInstance('romaneioItemsGrid');
    if (gridItems.datasource.data[0].ESTOQSUFICIENTE === 'N') {
      valid = false;
      text = 'Operação bloqueada! Há quantidade informada dos produtos não '
      + 'disponível no estoque. Verifique os itens em destaque.';
    } else if (gridItems.datasource.data[0].ESTOQEQUIVALENTE === 'S') {
      valid = false;
      text = 'Operação bloqueada! Quantidade dos itens equivalente a do estoque.';
    }

    if (!valid) {
      DialogService.show({
        name: 'estoqueInsuficiente',
        type: 'warning',
        title: 'Atenção',
        text,
        buttons: [
          {
            name: 'ok',
            component: 'ZdButton',
            label: 'OK',
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    }
    return valid;
  }

  public setFilterModalProdNEstoq() {
    const grid: Grid = Metadata.getInstance('gridProdNaoControlaEstoq');
    if (this.currentRow.NRINVENTRETI) {
      const conditions = [
        {
          name: 'P_CDFILIAL',
          operator: '=',
          value: this.currentRow.CDFILIAL,
        },
        {
          name: 'P_NRINVENTRETI',
          operator: '=',
          value: this.currentRow.NRINVENTRETI,
        },
      ];
  
      grid.datasource.filter = conditions;
      grid.datasource.get();
    }
  }

  public setFilterModalProdIrreg() {
    const grid: Grid = Metadata.getInstance('gridProdIrreg');
    if (this.currentRow.NRINVENTRETI) {
      const conditions = [
        {
          name: 'NRLISTA',
          operator: '=',
          value: this.nrlistaProd,
        },
      ];
  
      grid.datasource.filter = conditions;
      grid.datasource.get();
    }
  }
  
  public setFilterModalProdNaoPlanejado() {
    const grid: Grid = Metadata.getInstance('gridProdNaoPlanejado');
    const conditions = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: this.paramsAutoRemoval.CDFILIAL ? this.paramsAutoRemoval.CDFILIAL : null,
      },
      {
        name: 'NRINVENTRETI',
        operator: '=',
        value: this.paramsAutoRemoval.NRINVENTRETI ? this.paramsAutoRemoval.NRINVENTRETI : null,
      },
      {
        name: 'NRCONSPLAN',
        operator: '=',
        value: this.paramsAutoRemoval.NRCONSPLAN ? this.paramsAutoRemoval.NRCONSPLAN : null,
      },
    ];

    grid.datasource.filter = conditions;
    grid.datasource.get();
  }
  
  public setFilterModalProdIrregRetAuto({ component }: IEventParam<Grid>) {
    let type = 'I';
    if (component.name === 'prodSemPosEstoq') type = 'E';
    
    if (this.currentRow.NRINVENTRETI) {
      const conditions = [
        {
          name: 'NRLISTA',
          operator: '=',
          value: this.nrlistaProd,
        },
        {
          name: 'TIPO',
          operator: '=',
          value: type,
        },
      ];
  
      component.datasource.filter = conditions;
      component.datasource.get().then((result: IDictionary) => {
        if (result.length === 0) {
          const tab = Metadata.getInstance<Tabs>('tabsProds');
          if (type === 'I') tab.activeTab = 1;
        }
        this.updateStatusErrorItem(conditions);
      });
    }
  }
  
  private updateStatusErrorItem(conditions: IDictionary) {
    const agregada: Select = Metadata.getInstance('UNIDADE');
    const row = {
      CDFILIAL: agregada.value,
      NRINVENTRETI: this.currentRow.NRINVENTRETI,
      NRLISTA: conditions[0]?.value,
      TIPO: conditions[1]?.value,
    };
    Http.post('/updateStatusErrorItem', { requestType: 'Row', row }).then(() => {
      const grid: GridEditable = Metadata.getInstance('romaneioItemsGrid');
      grid.reload();
    }).catch((error) => {
      this.utilities.showAlert('error', error, 'error');
    });
  }

  private callAutoRemoval() {
    LoadingService.show(' ');
    this.paramsAutoRemoval.CDFILCENTPME = this.appController.getCdCentral();
    this.paramsAutoRemoval.DTPROCESSA = this.utilities.dtProcessamento;
    if (this.paramsAutoRemoval.checkUnplannedProd !== false) this.paramsAutoRemoval.checkUnplannedProd = true;
    this.paramsAutoRemoval.paramfilial = { ...this.paramfilial };
    this.paramsAutoRemoval.paramplaneja = { ...this.paramplaneja };
    this.paramsAutoRemoval.itemsRomaneio = this.allItemsRomaneio;
    Http.post('/autoRemoval', this.paramsAutoRemoval).then((response) => {
      const result = response.data.dataset?.params[0];
      // const servRet = response.data.dataset?.params[1];
      const gridRomaneio: GridEditable = Metadata.getInstance('romaneioGrid');
      if (result) {
        this.paramsAutoRemoval = result;
        if (result.ERROR && result.ERROR === 'Produtos Não Planejados') {
          this.dialogUnplannedProd();
        } else if (result.ERROR && result.ERROR === 'Não há lançamento de efetivos no período. '
        + 'Será realizado apenas o ajuste de inventário. Deseja continuar?') {
          this.dialogEfet();
        } else {
          this.nrlistaProd = result;
          this.openModalProdIrregRetAuto();
          this.utilities.showAlert('alert', 'Processo realizado com sucesso.', 'success');
        }
      } else {
        gridRomaneio.datasource.get().then(() => {
          // const gridItems: GridEditable = Metadata.getInstance('romaneioItemsGrid');
          // const currentRomaneio = gridRomaneio.datasource.currentRow;
          // gridRomaneio.datasource.currentRow = currentRomaneio;
          // gridItems.datasource.get();
          this.closeModalAutoRemoval();
          this.utilities.showAlert('alert', 'Processo realizado com sucesso.', 'success');

          // if (servRet) {
            
          // }
        });
      }
    }).catch((error: string) => {
      // if (error.indexOf('Produtos Não Planejados') >= 0) this.dialogUnplannedProd();
      // else
      this.utilities.showAlert('error', error, 'error');
    }).finally(() => {
      LoadingService.hide();
    });
  }

  private deleteConsplan(params: IDictionary) {
    Http.post('/deleteConsplanRetirada', params);
  }

  public closeItems() {
    const gridRomaneio: GridEditable = Metadata.getInstance('romaneioGrid');
    gridRomaneio.datasource.currentRow = {};
  }

  /*
  public findParamFilial() {
    const agregada: Select = Metadata.getInstance('UNIDADE');
    const row = {
      CDFILIAL: agregada.value,
    };
    
    Http.post('/findParamFilial', { requestType: 'Row', row }).then((response: IDictionary) => {
      const params = response.data.dataset.paramfilial[0];
      this.paramFilial = params;
    });
  }
  */
  
  public saveModalProdNaoPlanejado() {
    const grid: GridEditable = Metadata.getInstance('gridProdNaoPlanejado');
    const editedRows = grid.getEditedRows() as IDictionary;
    const rows = grid.datasource.data;
    rows.forEach((row: IDictionary) => {
      const editedRow = editedRows.find((el: IDictionary) => el.CDPRODUTO === row.CDPRODUTO);
      if (editedRow) row.CDSERVFILI = editedRow.CDSERVFILI;
    });
    this.paramsAutoRemoval.ERROR = null;
    this.paramsAutoRemoval.unplannedItems = rows;
    this.callAutoRemoval();
    this.closeModalProdNaoPlanejado();
  }
  
  public isDetailsActionVisible({ row }: IDictionary) {
    return row.IDORIGLANCPRINV === '1';
  }
  
  public isDetailsActionAlertVisible({ row }: IDictionary) {
    return row.IDCNTRESTOQ === 'N' || row.IDPRODATIV === 'I' || row.IDERRO !== null || row.IDERRO;
  }
  
  public errorMessage({ row }: IDictionary) {
    let message = '';
    if (row.IDCNTRESTOQ === 'N' && row.IDPRODATIV === 'I') {
      message = 'Produto não controla estoque e não está cadastrado para unidade ou inativo.';
    } else if (row.IDCNTRESTOQ === 'N') {
      message = 'Produto não controla estoque.';
    } else if (row.IDPRODATIV === 'I') {
      message = 'Produto não cadastrado para unidade ou inativo.';
    } else if (row.IDERRO === 'I') {
      message = 'Produto irregular.';
    } else if (row.IDERRO === 'E') {
      message = 'Produto sem posição de estoque.';
    }
    return message;
  }
  
  public checkDisableGridAdd = (this.gridEditController.newRow || this.disableGridAdd);
}
